<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

use Invertus\SwedBank\Config\Config;
use Invertus\SwedBank\Entity\SwedbankPayment;
use Invertus\SwedBank\Service\OrderHandler;
use Invertus\SwedBank\Util\CartValidation;
use Invertus\SwedBank\Util\OrderValidation;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\Action;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\BillingDetails;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\CustomerDetails;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\DynamicData;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\HPSTxn;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\MerchantConfiguration;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\PersonalDetails;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\RiskDetails;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\ShippingDetails;
use SwedbankPaymentPortal\SharedEntity\Authentication;
use SwedbankPaymentPortal\SwedbankPaymentPortal;
use SwedbankPaymentPortal\SharedEntity\Amount;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\SetupRequest;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction;
use SwedbankPaymentPortal\CC\Type\ScreeningAction;
use SwedbankPaymentPortal\CC\Type\TransactionChannel;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\TxnDetails;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\ThreeDSecure;
use SwedbankPaymentPortal\CC\HPSCommunicationEntity\SetupRequest\Transaction\CardTxn;
use SwedbankPaymentPortal\UrlCallback;

class SwedBankHpsModuleFrontController extends ModuleFrontController
{
    const FILENAME = 'hps';

    /**
     * @var SwedBank
     */
    public $module;

    public function checkAccess()
    {
        $module = $this->module;
        $customer = $this->context->customer;

        $isPaymentProcess = Tools::isSubmit('id_order');

        $link = $this->context->link;
        $redirectUrl = $isPaymentProcess ?
            $link->getPageLink('history') :
            $link->getPageLink('order');

        if (!$module->active || !$module->isConfigured()) {
            Tools::redirect($redirectUrl);
        }

        if ($isPaymentProcess) {
            $idOrder = Tools::getValue('id_order');
            $order = new Order($idOrder);
            /** @var OrderValidation $orderValidation */
            $orderValidation = $module->get(OrderValidation::class);
            $isAccessGranted = $orderValidation->validate($order, $customer);
        } else {
            /** @var CartValidation $cartValidation */
            $cartValidation = $module->get(CartValidation::class);
            $isAccessGranted = $cartValidation->validate($this->context->cart, $customer);
        }

        if (!$isAccessGranted) {
            Tools::redirect($redirectUrl);
        }

        return true;
    }

    public function postProcess()
    {
        $link = $this->context->link;
        $moduleName = $this->module->name;

        /** @var SwedbankPaymentPortal $paymentPortal */
        $paymentPortal = $this->module->get(SwedbankPaymentPortal::class);
        /** @var OrderHandler $orderService */
        $orderService = $this->module->get(OrderHandler::class);
        $paymentName = $this->module->l('Card', self::FILENAME);

        if (Tools::isSubmit('id_order')) {
            $idOrder = Tools::getValue('id_order');
        } else {
            $idOrder = $orderService->processOrderCreate($this->context->cart, $this->context->currency, $paymentName);
        }

        $order = new Order($idOrder);

        if ($paymentName != $order->payment) {
            $order->payment = $paymentName;
            $order->save();
        }

        $customer = $this->context->customer;
        $shopUrl = $orderService->getOrderDetailLink($customer, $order, $this->context->language->id);
        $shopUrl1 = parse_url($shopUrl);
        $shopUrl1 =  (!empty($shopUrl1['scheme']) ? $shopUrl1['scheme'] : 'http' )."://".$shopUrl1['host'];
        $customerDetails = $this->getCustomerDetails($order);
        $merchantConfig = new MerchantConfiguration(TransactionChannel::web(), $shopUrl);
        $riskAction = new Action(ScreeningAction::preAuthorization(), $merchantConfig, $customerDetails);

        $merchantReference = $orderService->formatMerchantReference($order, $this->context->shop->id);

        $txnDetails = new TxnDetails(
            $riskAction,
            $merchantReference,
            new Amount($order->total_paid_tax_incl),
            new ThreeDSecure(
                sprintf($this->module->l('Order nr. %d', 'hps'), $order->id),
                $shopUrl1,
                new DateTime()
            )
        );

        $params = ['id_order' => $order->id];

        $confirmUrl = $link->getModuleLink($moduleName, 'hpsconfirm', $params);
        $pageSetId = (int) $orderService->getPaymentLanguage($order->id_lang, $order->id_shop, 'card');

        $dynamicData = new DynamicData(null, $shopUrl);
        $hpsTxn = new HPSTxn($confirmUrl, $confirmUrl, $confirmUrl, $pageSetId, $dynamicData);

        /** @var Authentication $authentication */
        $authentication = $this->module->get(Authentication::class);

        $transaction  = new Transaction($txnDetails, $hpsTxn, new CardTxn());
        $setupRequest = new SetupRequest($authentication, $transaction);

        $callbackParams = ['secret_key' => Tools::encrypt($moduleName), 'id_order' => $order->id];
        $callbackUrl = $link->getModuleLink($moduleName, 'hpscallback', $callbackParams);

        try {
            $response = $paymentPortal
                ->getPaymentCardHostedPagesGateway()
                ->initPayment($setupRequest, UrlCallback::create($callbackUrl));

            $url = $response->getCustomerRedirectUrl();

            Tools::redirect($url);
        } catch (Exception $e) {
            $this->errors = $this
                ->module->l('Operation failed. Please choose another payment method or try again.', self::FILENAME);

            $confirmationUrl = $orderService->getOrderConfirmationUrl($order, $this->context->language->id);
            $this->redirectWithNotifications($confirmationUrl);
        }
    }

    /**
     * @param Order $order
     *
     * @return CustomerDetails
     */
    private function getCustomerDetails(Order $order)
    {
        $idLang = $this->context->language->id;
        $customer = new Customer($order->id_customer);

        $shippingAddress = new Address($order->id_address_delivery);
        $shippingCountry = new Country($shippingAddress->id_country, $idLang);
        $billingAddress = new Address($order->id_address_invoice);
        $billingCountry = new Country($billingAddress->id_country, $idLang);

        $customerFullName = sprintf('%s %s', $billingAddress->firstname, $billingAddress->lastname);

        $billingDetails = new BillingDetails(
            '',
            $customerFullName,
            $billingAddress->postcode,
            $billingAddress->address1,
            $billingAddress->address2,
            $billingAddress->city,
            $billingCountry->iso_code
        );

        $personalDetails = new PersonalDetails(
            $customer->firstname,
            $customer->lastname,
            $shippingAddress->phone_mobile ? $shippingAddress->phone_mobile : $shippingAddress->phone
        );

        $shippingDetails = new ShippingDetails(
            '',
            $shippingAddress->firstname,
            $shippingAddress->lastname,
            $shippingAddress->address1,
            $shippingAddress->address2,
            $shippingAddress->city,
            $shippingCountry->iso_code,
            $shippingAddress->postcode
        );

        $riskDetails = new RiskDetails(Tools::getRemoteAddr(), $customer->email);
        $customerDetails = new CustomerDetails($billingDetails, $personalDetails, $shippingDetails, $riskDetails);

        return $customerDetails;
    }
}
